from enum import Enum


class ValidationError(Exception):
    pass


class SeqMetadataKeys(str, Enum):
    MELDETATBESTAND = "MELDETATBESTAND"
    SPECIES = "SPECIES"
    LAB_SEQUENCE_ID = "LAB_SEQUENCE_ID"
    DEMIS_NOTIFICATION_ID = "DEMIS_NOTIFICATION_ID"
    STATUS = "STATUS"
    IGS_ID = "IGS_ID"
    DATE_OF_SAMPLING = "DATE_OF_SAMPLING"
    DATE_OF_SEQUENCING = "DATE_OF_SEQUENCING"
    DATE_OF_SUBMISSION = "DATE_OF_SUBMISSION"
    SEQUENCING_INSTRUMENT = "SEQUENCING_INSTRUMENT"
    SEQUENCING_PLATFORM = "SEQUENCING_PLATFORM"
    ADAPTER = "ADAPTER"
    SEQUENCING_STRATEGY = "SEQUENCING_STRATEGY"
    ISOLATION_SOURCE = "ISOLATION_SOURCE"
    HOST = "HOST"
    SEQUENCING_REASON = "SEQUENCING_REASON"
    GEOGRAPHIC_LOCATION = "GEOGRAPHIC_LOCATION"
    ISOLATE = "ISOLATE"
    AUTHOR = "AUTHOR"
    NAME_AMP_PROTOCOL = "NAME_AMP_PROTOCOL"
    PRIMER_SCHEME = "PRIMER_SCHEME"
    PRIME_DIAGNOSTIC_LAB_DEMIS_LAB_ID = "PRIME_DIAGNOSTIC_LAB.DEMIS_LAB_ID"
    PRIME_DIAGNOSTIC_LAB_NAME = "PRIME_DIAGNOSTIC_LAB.NAME"
    PRIME_DIAGNOSTIC_LAB_ADDRESS = "PRIME_DIAGNOSTIC_LAB.STREET"
    PRIME_DIAGNOSTIC_LAB_POSTAL_CODE = "PRIME_DIAGNOSTIC_LAB.POSTAL_CODE"
    PRIME_DIAGNOSTIC_LAB_FEDERAL_STATE = "PRIME_DIAGNOSTIC_LAB.FEDERAL_STATE"
    SEQUENCING_LAB_DEMIS_LAB_ID = "SEQUENCING_LAB.DEMIS_LAB_ID"
    SEQUENCING_LAB_NAME = "SEQUENCING_LAB.NAME"
    SEQUENCING_LAB_ADDRESS = "SEQUENCING_LAB.STREET"
    SEQUENCING_LAB_POSTAL_CODE = "SEQUENCING_LAB.POSTAL_CODE"
    SEQUENCING_LAB_FEDERAL_STATE = "SEQUENCING_LAB.FEDERAL_STATE"
    UPLOADS = "Uploads"
    REPOSITORY_NAME = "REPOSITORY_NAME"
    REPOSITORY_LINK = "REPOSITORY_LINK"
    REPOSITORY_ID = "REPOSITORY_ID"
    UPLOAD_DATE = "UPLOAD_DATE"
    UPLOAD_STATUS = "UPLOAD_STATUS"
    UPLOAD_SUBMITTER = "UPLOAD_SUBMITTER"
    FILES = "Files"
    FILE_NAME = "FILE_NAME"
    FILE_SHA256SUM = "FILE_SHA256SUM"


seq_metadata_schema = {
    "type": "object",
    "properties": {
        SeqMetadataKeys.MELDETATBESTAND: {
            "enum": [
                "EHCP",
                "LISP",
                "SALP",
                "STYP",
                "INVP",
                "NEIP",
                "MSVP",
                "MYTP",
                "CVDP",
                "HIVP",
                "NEGP",
                "EBCP",
                "ACBP",
                "CDFP",
                "MRAP",
                "SALP",
                "HEVP",
                "HAVP",
                "LEGP",
                "SPNP",
                "WNVP",
            ],
        },
        SeqMetadataKeys.SPECIES: {"type": "string", "pattern": "[A-z0-9\\-_]*"},
        SeqMetadataKeys.LAB_SEQUENCE_ID: {
            "type": "string",
            "pattern": "[A-z,0-9,-,_]*",
        },
        SeqMetadataKeys.DEMIS_NOTIFICATION_ID: {"type": "string"},
        SeqMetadataKeys.STATUS: {"type": "string"},
        SeqMetadataKeys.IGS_ID: {
            "type": "string",
            "pattern": "(IMS|IGS)-[0-9]{5}-[A-Z]{4}-[A-z0-9\\-_]+",
        },
        SeqMetadataKeys.DATE_OF_SAMPLING: {"$ref": "#/$defs/date"},
        SeqMetadataKeys.DATE_OF_SEQUENCING: {"$ref": "#/$defs/date"},
        SeqMetadataKeys.DATE_OF_SUBMISSION: {"$ref": "#/$defs/date"},
        SeqMetadataKeys.SEQUENCING_INSTRUMENT: {
            "enum": [
                "454_GS",
                "454_GS_20",
                "454_GS_FLX",
                "454_GS_FLX+",
                "454_GS_FLX_Titanium",
                "454_GS_Junior",
                "HiSeq_X_Five",
                "HiSeq_X_Ten",
                "Illumina_Genome_Analyzer",
                "Illumina_Genome_Analyzer_II",
                "Illumina_Genome_Analyzer_IIx",
                "Illumina_HiScanSQ",
                "Illumina_HiSeq_1000",
                "Illumina_HiSeq_1500",
                "Illumina_HiSeq_2000",
                "Illumina_HiSeq_2500",
                "Illumina_HiSeq_3000",
                "Illumina_HiSeq_4000",
                "Illumina_iSeq_100",
                "Illumina_MiSeq",
                "Illumina_MiniSeq",
                "Illumina_NovaSeq_6000",
                "NextSeq_500",
                "NextSeq_550",
                "PacBio_RS",
                "PacBio_RS_II",
                "Sequel",
                "Ion_Torrent_PGM",
                "Ion_Torrent_Proton",
                "Ion_Torrent_S5",
                "Ion_Torrent_S5_XL",
                "AB_3730xL_Genetic_Analyzer",
                "AB_3730_Genetic_Analyzer",
                "AB_3500xL_Genetic_Analyzer",
                "AB_3500_Genetic_Analyzer",
                "AB_3130xL_Genetic_Analyzer",
                "AB_3130_Genetic_Analyzer",
                "AB_310_Genetic_Analyzer",
                "MinION",
                "GridION",
                "PromethION",
                "BGISEQ-500",
                "DNBSEQ-T7",
                "DNBSEQ-G400",
                "DNBSEQ-G50",
                "DNBSEQ-G400_FAST",
                "unspecified",
            ],
        },
        SeqMetadataKeys.SEQUENCING_PLATFORM: {
            "enum": [
                "LS454",
                "ILLUMINA",
                "PACBIO_SMRT",
                "ION_TORRENT",
                "CAPILLARY",
                "OXFORD_NANOPORE",
                "BGISEQ",
                "DNBSEQ",
            ],
        },
        SeqMetadataKeys.ADAPTER: {
            "type": "string",
            "pattern": "[a-zA-Z0-9_\\- ,]+",
        },
        SeqMetadataKeys.SEQUENCING_STRATEGY: {
            "enum": [
                "WGS",
                "WGA",
                "WXS",
                "RNA-Seq",
                "ssRNA-seq",
                "miRNA-Seq",
                "ncRNA-Seq",
                "FL-cDNA",
                "EST",
                "Hi-C",
                "ATAC-seq",
                "WCS",
                "RAD-Seq",
                "CLONE",
                "POOLCLONE",
                "AMPLICON",
                "CLONEEND",
                "FINISHING",
                "ChIP-Seq",
                "MNase-Seq",
                "DNase-Hypersensitivity",
                "Bisulfite-Seq",
                "CTS",
                "MRE-Seq",
                "MeDIP-Seq",
                "MBD-Seq",
                "Tn-Seq",
                "VALIDATION",
                "FAIRE-seq",
                "SELEX",
                "RIP-Seq",
                "ChIA-PET",
                "Synthetic-Long-Read",
                "Targeted-Capture",
                "Tethered Chromatin Conformation Capture",
                "OTHER",
            ],
        },
        SeqMetadataKeys.ISOLATION_SOURCE: {
            "enum": [
                "Air sample",
                "Ear sample",
                "Pus sample",
                "Bile sample",
                "Tick sample",
                "Milk sample",
                "Hair sample",
                "Vein sample",
                "Lens sample",
                "Tear sample",
                "Skin sample",
                "Nail sample",
                "Water sample",
                "Serum sample",
                "Joint sample",
                "Smear sample",
                "Feces sample",
                "Sperm sample",
                "Stone sample",
                "Vomit sample",
                "Lymph sample",
                "Mucus sample",
                "Gonad sample",
                "Larva sample",
                "Cream sample",
                "Bursa sample",
                "Sebum sample",
                "Fluid sample",
                "Blood sample",
                "Urine sample",
                "Louse sample",
                "Lesion sample",
                "Tendon sample",
                "Fascia sample",
                "Living sample",
                "Maggot sample",
                "Plasma sample",
                "Lochia sample",
                "Breast sample",
                "Artery sample",
                "Biopsy sample",
                "Insect sample",
                "Liquor sample",
                "Sputum sample",
                "Neuroma sample",
                "Genetic sample",
                "Genital sample",
                "Exudate sample",
                "Cytology sample",
                "Synovial sample",
                "Ear swab sample",
                "Thrombus sample",
                "Parasite sample",
                "Soy milk sample",
                "Platelet sample",
                "Prostate sample",
                "Synovium sample",
                "Hematoma sample",
                "Ligament sample",
                "Appendix sample",
                "Effusion sample",
                "Helminth sample",
                "Meconium sample",
                "Cartilage sample",
                "Eye fluid sample",
                "Pituitary sample",
                "Colostrum sample",
                "Hydrocele sample",
                "Colectomy sample",
                "Gallstone sample",
                "Endocrine sample",
                "Skin cyst sample",
                "Hair bulb sample",
                "Dialysate sample",
                "Hot stool sample",
                "Ice-cream sample",
                "Foodstuff sample",
                "Wash fluid sample",
                "Urological sample",
                "Cyst fluid sample",
                "Skin scale sample",
                "Whole milk sample",
                "Cord blood sample",
                "Body fluid sample",
                "Epididymis sample",
                "Anal polyp sample",
                "Lip biopsy sample",
                "Blood film sample",
                "Polar body sample",
                "Anal smear sample",
                "Cystectomy sample",
                "Spot urine sample",
                "Transudate sample",
                "Lymph node sample",
                "Cow's milk sample",
                "Ear tissue sample",
                "Eye tissue sample",
                "Fibroblast sample",
                "Soft stool sample",
                "Paint chip sample",
                "Blood clot sample",
                "Wound fluid sample",
                "Lung biopsy sample",
                "Skin lesion sample",
                "Nasal polyp sample",
                "Omentectomy sample",
                "Breast duct sample",
                "Bursa fluid sample",
                "Sinus fluid sample",
                "Whole blood sample",
                "Goat's milk sample",
                "Labia ulcer sample",
                "Gastrectomy sample",
                "Bone biopsy sample",
                "Parathyroid sample",
                "Oral cavity sample",
                "Colon strip sample",
                "Anal biopsy sample",
                "Fecal fluid sample",
                "Edema fluid sample",
                "Oral saliva sample",
                "Labia smear sample",
                "Dental cyst sample",
                "Skin biopsy sample",
                "Soft tissue sample",
                "Oral tissue sample",
                "Nephrectomy sample",
                "Nasal fluid sample",
                "Bone tissue sample",
                "Rectal snip sample",
                "Renal stone sample",
                "Heart valve sample",
                "Respiratory sample",
                "Anal tissue sample",
                "Bone marrow sample",
                "Breast cyst sample",
                "Nerve tissue sample",
                "Orchidectomy sample",
                "Mouth biopsy sample",
                "Renal biopsy sample",
                "Penis tissue sample",
                "Formula milk sample",
                "Excised bone sample",
                "Cecum biopsy sample",
                "Reticulocyte sample",
                "Venous blood sample",
                "Liver tissue sample",
                "Colon tissue sample",
                "Bursa tissue sample",
                "Bursa biopsy sample",
                "Joint biopsy sample",
                "Nerve biopsy sample",
                "Vas deferens sample",
                "Excised lung sample",
                "Laryngectomy sample",
                "Liver biopsy sample",
                "Ileal biopsy sample",
                "Tumor tissue sample",
                "Buccal smear sample",
                "Hemofiltrate sample",
                "Male genital sample",
                "Breast fluid sample",
                "Thyroid lobe sample",
                "Nasal biopsy sample",
                "Rectal polyp sample",
                "Ovarian cyst sample",
                "Hysterectomy sample",
                "Tissue fluid sample",
                "Skimmed milk sample",
                "Sheep's milk sample",
                "Aorta tissue sample",
                "Kidney tissue sample",
                "Lipoma biopsy sample",
                "Excised bursa sample",
                "Fascia biopsy sample",
                "Testes biopsy sample",
                "Pleura biopsy sample",
                "Ureter biopsy sample",
                "Muscle biopsy sample",
                "Testes tissue sample",
                "Uterus tissue sample",
                "Cervix tissue sample",
                "Aqueous humor sample",
                "Chylous fluid sample",
                "Empyema fluid sample",
                "Jejunal fluid sample",
                "Biliary stone sample",
                "Bladder stone sample",
                "Amniotic cell sample",
                "Clotted blood sample",
                "Blister fluid sample",
                "Thyroid gland sample",
                "Head of femur sample",
                "Thyroid fluid sample",
                "Larynx biopsy sample",
                "Dental tissue sample",
                "Tonsil biopsy sample",
                "Rectal tissue sample",
                "Ascitic fluid sample",
                "Embryo biopsy sample",
                "Fistula fluid sample",
                "Genitourinary sample",
                "Tendon biopsy sample",
                "Adrenal gland sample",
                "Tongue biopsy sample",
                "Tonsillectomy sample",
                "Gastric ulcer sample",
                "Colonic polyp sample",
                "Rectal biopsy sample",
                "Ovarian fluid sample",
                "Parotidectomy sample",
                "Rectal scrape sample",
                "Gastric fluid sample",
                "Thyroidectomy sample",
                "Maternal milk sample",
                "Breast biopsy sample",
                "Desmoid tumor sample",
                "Treated water sample",
                "24 hour urine sample",
                "Red blood cell sample",
                "Adrenal biopsy sample",
                "Gastric biopsy sample",
                "Colonic biopsy sample",
                "Synovial fluid sample",
                "Adrenal tissue sample",
                "Cervical smear sample",
                "Adipose tissue sample",
                "Oral secretion sample",
                "Dialysis fluid sample",
                "Duodenal fluid sample",
                "Gastric tissue sample",
                "Mastoid biopsy sample",
                "Omentum biopsy sample",
                "Jejunal biopsy sample",
                "Anal resection sample",
                "Vaginal biopsy sample",
                "Omentum tissue sample",
                "Ureteric stone sample",
                "Bone and joint sample",
                "Gingiva biopsy sample",
                "Body substance sample",
                "Female genital sample",
                "Excised tendon sample",
                "Excised fascia sample",
                "Excised rectum sample",
                "Bladder biopsy sample",
                "Urethra biopsy sample",
                "Cervical polyp sample",
                "Pancreatectomy sample",
                "Vitreous humor sample",
                "Drainage fluid sample",
                "Cardiovascular sample",
                "Dermatological sample",
                "Ovarian tissue sample",
                "Vaginal tissue sample",
                "Arterial blood sample",
                "Ovarian biopsy sample",
                "Duodenal ulcer sample",
                "Capillary blood sample",
                "Heart sac fluid sample",
                "Skin curettings sample",
                "Musculoskeletal sample",
                "Placenta biopsy sample",
                "Hydrocele fluid sample",
                "Prostatic fluid sample",
                "Pancreas tissue sample",
                "Synovium biopsy sample",
                "Prostate biopsy sample",
                "Cholecystectomy sample",
                "Pancreas biopsy sample",
                "Cervical biopsy sample",
                "Temporal artery sample",
                "Total colectomy sample",
                "Foreskin biopsy sample",
                "Chorionic villi sample",
                "Epidural lesion sample",
                "Ligament biopsy sample",
                "Tracheal biopsy sample",
                "Duodenal biopsy sample",
                "Ileal resection sample",
                "Rib bone marrow sample",
                "Colostomy fluid sample",
                "Bronchial fluid sample",
                "Ileostomy fluid sample",
                "Skin fibroblast sample",
                "Acidified serum sample",
                "Prostate tissue sample",
                "Untreated water sample",
                "Abdominal fluid sample",
                "Sputum - coughed sample",
                "Stomach aspirate sample",
                "First void urine sample",
                "Bronchial biopsy sample",
                "Urological fluid sample",
                "Placental tissue sample",
                "Middle ear fluid sample",
                "Fecal concretion sample",
                "Mid-stream urine sample",
                "White blood cell sample",
                "Kidney brushings sample",
                "Pancreatic fluid sample",
                "Excised ligament sample",
                "Bile duct biopsy sample",
                "Subretinal fluid sample",
                "Corneal scraping sample",
                "Pancreatic stone sample",
                "Bone marrow clot sample",
                "Intraocular lens sample",
                "Brain cyst fluid sample",
                "Spleen resection sample",
                "Cartilage biopsy sample",
                "Articular tissue sample",
                "Peritoneal fluid sample",
                "Gastrointestinal sample",
                "Total gastrectomy sample",
                "Respiratory fluid sample",
                "Penis skin biopsy sample",
                "Lymph node tissue sample",
                "Middle ear biopsy sample",
                "Peritoneal biopsy sample",
                "Esophageal biopsy sample",
                "Gastric brushings sample",
                "Endometrial polyp sample",
                "Skin punch biopsy sample",
                "Semi-skimmed milk sample",
                "Esophageal tissue sample",
                "Excised cartilage sample",
                "Resected prostate sample",
                "Vocal cord biopsy sample",
                "Pharyngeal biopsy sample",
                "Lumpectomy breast sample",
                "Vaginal secretion sample",
                "Upper respiratory sample",
                "Extradural lesion sample",
                "Lymph node biopsy sample",
                "Lung direct smear sample",
                "Partial colectomy sample",
                "Total nephrectomy sample",
                "Breast cyst fluid sample",
                "Postmortem tissue sample",
                "Lower respiratory sample",
                "Surgical excision sample",
                "Urological tissue sample",
                "Second void urine sample",
                "Endotracheal fluid sample",
                "Vulval skin biopsy sample",
                "Parathyroid biopsy sample",
                "Liver direct smear sample",
                "Small bowel tissue sample",
                "Renal pelvis fluid sample",
                "Excised mesorectum sample",
                "Endometrial biopsy sample",
                "Native heart valve sample",
                "Conjunctival smear sample",
                "Nasal lavage fluid sample",
                "Endocardium biopsy sample",
                "Soft tissue biopsy sample",
                "Thyroid cyst fluid sample",
                "Parathyroid tissue sample",
                "Bladder curettings sample",
                "Large bowel tissue sample",
                "Ureteric brushings sample",
                "Endocervical smear sample",
                "Papanicolaou smear sample",
                "Duodenal brushings sample",
                "Endocervical polyp sample",
                "Peritoneal dialysis sample",
                "Amniocentesis fluid sample",
                "Partial nephrectomy sample",
                "Sternal bone marrow sample",
                "Cerebrospinal fluid sample",
                "Transtracheal fluid sample",
                "Early morning urine sample",
                "Uterine cervix cyst sample",
                "Thoracentesis fluid sample",
                "Right hemicolectomy sample",
                "Excised parathyroid sample",
                "Bronchial brushings sample",
                "Vaginal vault smear sample",
                "Breast direct smear sample",
                "Suprapubic aspirate sample",
                "Total parotidectomy sample",
                "Pancreas cyst fluid sample",
                "Cerebellar neoplasm sample",
                "Core sample of tissue block",
                "Thyroid direct smear sample",
                "Esophageal brushings sample",
                "Sigmoid colon biopsy sample",
                "Biliary tract tissue sample",
                "Skin excision biopsy sample",
                "Branchial cleft cyst sample",
                "Minor salivary gland sample",
                "Bone marrow trephine sample",
                "Nasal sinus contents sample",
                "Vaginal vault biopsy sample",
                "Skin incision biopsy sample",
                "Segmentectomy breast sample",
                "Red blood cell fluid sample",
                "Parotid gland saliva sample",
                "Bone marrow fragment sample",
                "Female genital fluid sample",
                "Breast tru-cut biopsy sample",
                "Frozen section breast sample",
                "Excised adrenal gland sample",
                "Nasopharyngeal biopsy sample",
                "Gastric antral biopsy sample",
                "Gastric fundus biopsy sample",
                "Sigmoid colonic polyp sample",
                "Neck block dissection sample",
                "Vaginal washout fluid sample",
                "Machine hemodialysate sample",
                "Brain neoplasm tissue sample",
                "Frozen tissue section sample",
                "Small bowel resection sample",
                "Endometrial resection sample",
                "Deoxyribonucleic acid sample",
                "Contact lens solution sample",
                "Female genital tissue sample",
                "Fallopian tube biopsy sample",
                "Umbilical cord tissue sample",
                "Endometrial curettings sample",
                "Urinary bladder tissue sample",
                "Frozen section bladder sample",
                "Cervical loop excision sample",
                "Ascending colon biopsy sample",
                "Upper respiratory swab sample",
                "Excised salivary gland sample",
                "Gastrointestinal fluid sample",
                "Retroperitoneal tissue sample",
                "Ileocecal valve biopsy sample",
                "Gingivocrevicular fluid sample",
                "Pancreaticoduodenectomy sample",
                "Gastric lavage aspirate sample",
                "Renal transplant biopsy sample",
                "Upper respiratory fluid sample",
                "Prostatic massage fluid sample",
                "Oral mucosal transudate sample",
                "Prostate tru-cut biopsy sample",
                "Lower respiratory fluid sample",
                "Sigmoid colon brushings sample",
                "Breast nipple discharge sample",
                "Iliac crest bone marrow sample",
                "Tissue impression smear sample",
                "Ampulla of Vater biopsy sample",
                "Gastrointestinal tissue sample",
                "Descending colon biopsy sample",
                "Thyroid neoplasm tissue sample",
                "Terminal ileum resection sample",
                "Upper respiratory tissue sample",
                "Lower respiratory tissue sample",
                "Excised sublingual gland sample",
                "Superficial parotidectomy sample",
                "Resected articular tissue sample",
                "Frozen section lymph node sample",
                "Placental membrane tissue sample",
                "Lung fine needle aspirate sample",
                "Cerebrospinal fluid smear sample",
                "Left colic flexure biopsy sample",
                "Rectosigmoid colon biopsy sample",
                "Intravenous infusion fluid sample",
                "Right colic flexure biopsy sample",
                "Common bile duct brushings sample",
                "Transbronchial lung biopsy sample",
                "Excised submandibular gland sample",
                "Transthoracic needle biopsy sample",
                "Breast fine needle aspirate sample",
                "Thyroid fine needle aspirate sample",
                "Anterior resection of rectum sample",
                "Bronchoalveolar lavage fluid sample",
                "Products of conception tissue sample",
                "Excised breast ectopic tissue sample",
                "Intravenous lipid infusion fluid sample",
                "Central nervous system cyst fluid sample",
                "Donor-organ preservation solution sample",
                "Cardiopulmonary bypass pump fluid sample",
                "Colorectal junction neoplasm tissue sample",
                "Vertebral spinous process bone marrow sample",
                "Soft tissue lesion fine needle aspirate sample",
                "Frozen section skin sample obtained by excision",
                "Hysterectomy and left salpingo-oophorectomy sample",
                "Excised uterus, fallopian tubes and ovaries sample",
                "Hydrocele sample obtained by fine needle aspiration",
                "Hysterectomy and right salpingo-oophorectomy sample",
                "Tissue sample from aneurysm obtained by surgical resection",
                "Screening - procedure intent",
            ],
        },
        SeqMetadataKeys.HOST: {"enum": ["Homo sapiens"]},
        SeqMetadataKeys.SEQUENCING_REASON: {"enum": ["random", "requested", "other", "clinical"]},
        SeqMetadataKeys.GEOGRAPHIC_LOCATION: {
            "type": "string",
            "pattern": "[0-9]{3}$",
        },
        SeqMetadataKeys.ISOLATE: {
            "type": "string",
            "pattern": "[a-zA-Z0-9_\\- ,]*",
        },
        SeqMetadataKeys.AUTHOR: {
            "type": "string",
            "pattern": "[a-zA-Z0-9_\\- ,]*",
        },
        SeqMetadataKeys.NAME_AMP_PROTOCOL: {
            "type": "string",
            "pattern": "[a-zA-Z0-9_\\- ,]*",
        },
        SeqMetadataKeys.PRIME_DIAGNOSTIC_LAB_DEMIS_LAB_ID: {
            "type": "string",
            "pattern": r"\bDEMIS-[0-9]{5}\b",
        },
        SeqMetadataKeys.PRIME_DIAGNOSTIC_LAB_NAME: {
            "type": "string",
            "pattern": "[a-zA-Z0-9_\\- ,]*",
        },
        SeqMetadataKeys.PRIME_DIAGNOSTIC_LAB_ADDRESS: {
            "type": "string",
            "pattern": "[a-zA-Z0-9_\\- ,]*",
        },
        SeqMetadataKeys.PRIME_DIAGNOSTIC_LAB_POSTAL_CODE: {
            "type": "string",
            "pattern": "[0-9]{5}|^$",
        },
        SeqMetadataKeys.PRIME_DIAGNOSTIC_LAB_FEDERAL_STATE: {"$ref": "#/$defs/federalStates"},
        SeqMetadataKeys.PRIMER_SCHEME: {
            "type": "string",
            "pattern": "[a-zA-Z0-9_\\- ,]*",
        },
        SeqMetadataKeys.SEQUENCING_LAB_DEMIS_LAB_ID: {
            "type": "string",
            "pattern": r"\bDEMIS-[0-9]{5}\b",
        },
        SeqMetadataKeys.SEQUENCING_LAB_NAME: {
            "type": "string",
            "pattern": "[a-zA-Z0-9_\\- ,]*",
        },
        SeqMetadataKeys.SEQUENCING_LAB_ADDRESS: {
            "type": "string",
            "pattern": "[a-zA-Z0-9_\\- ,]*",
        },
        SeqMetadataKeys.SEQUENCING_LAB_POSTAL_CODE: {
            "type": "string",
            "pattern": "[0-9]{5}$",
        },
        SeqMetadataKeys.SEQUENCING_LAB_FEDERAL_STATE: {"$ref": "#/$defs/federalStates"},
        SeqMetadataKeys.UPLOADS: {
            "type": "array",
            "items": {
                "type": "object",
                "properties": {
                    SeqMetadataKeys.REPOSITORY_NAME: {"enum": ["GISAID", "ENA", "SRA", "PubMLST", "GenBank", "Other"]},
                    SeqMetadataKeys.REPOSITORY_LINK: {"type": "string"},
                    SeqMetadataKeys.REPOSITORY_ID: {"type": "string"},
                    SeqMetadataKeys.UPLOAD_DATE: {"$ref": "#/$defs/date"},
                    SeqMetadataKeys.UPLOAD_STATUS: {"enum": ["Accepted", "Planned", "Denied", "Other"]},
                    SeqMetadataKeys.UPLOAD_SUBMITTER: {"type": "string"},
                },
                "additionalProperties": False,
                "required": [
                    SeqMetadataKeys.REPOSITORY_NAME,
                    SeqMetadataKeys.REPOSITORY_ID,
                    SeqMetadataKeys.UPLOAD_STATUS,
                ],
            },
            "minItems": 1,
        },
        SeqMetadataKeys.FILES: {
            "type": "array",
            "items": {
                "type": "object",
                "properties": {
                    SeqMetadataKeys.FILE_NAME: {
                        "type": "string",
                        "pattern": "^[a-zA-Z0-9_\\-]+.(fasta|fastq.gz)",
                    },
                    SeqMetadataKeys.FILE_SHA256SUM: {
                        "type": "string",
                        "pattern": "[A-Fa-f0-9]{64}",
                    },
                },
                "required": ["FILE_NAME", "FILE_SHA256SUM"],
                "additionalProperties": False,
            },
            "minItems": 1,
            "uniqueItems": True,
        },
    },
    "$defs": {
        "date": {
            "type": "string",
            "format": "date",
            "pattern": "(((19|20)([2468][048]|[13579][26]|0[48])|2000)[/-]02[/-]29|((19|20)[0-9]{2}[/-](0[469]|11)[/-](0[1-9]|[12][0-9]|30)|(19|20)[0-9]{2}[/-](0[13578]|1[02])[/-](0[1-9]|[12][0-9]|3[01])|(19|20)[0-9]{2}[/-]02[/-](0[1-9]|1[0-9]|2[0-8]{1})))",  # noqa: E501
            "minLength": 10,
            "maxLength": 10,
        },
        "federalStates": {
            "enum": [
                "Baden-Württemberg",
                "Bayern",
                "Berlin",
                "Brandenburg",
                "Bremen",
                "Hamburg",
                "Hessen",
                "Mecklenburg-Vorpommern",
                "Niedersachsen",
                "Nordrhein-Westfalen",
                "Rheinland-Pfalz",
                "Saarland",
                "Sachsen",
                "Sachsen-Anhalt",
                "Schleswig-Holstein",
                "Thüringen",
            ],
        },
    },
    "required": [
        SeqMetadataKeys.MELDETATBESTAND,
        SeqMetadataKeys.LAB_SEQUENCE_ID,
        SeqMetadataKeys.DATE_OF_SUBMISSION,
        SeqMetadataKeys.SEQUENCING_INSTRUMENT,
        SeqMetadataKeys.SEQUENCING_PLATFORM,
        SeqMetadataKeys.HOST,
        SeqMetadataKeys.SEQUENCING_REASON,
        SeqMetadataKeys.SEQUENCING_LAB_DEMIS_LAB_ID,
    ],
}
